@@ -1,17 +1,19 @@
 mod core;
 mod entry;
 mod file;
+mod header;
 mod index;
 
+pub(crate) use self::core::BLOB_INDEX_FILE_EXTENSION;
 pub(crate) use self::core::{Blob, FileName};
 pub use self::entry::Entry;
 pub(crate) use self::file::File;
 pub use self::index::bloom as filter;
-pub(crate) use self::index::Config as BloomConfig;
+pub(crate) use self::index::{BloomConfig, IndexConfig};
 pub(crate) use super::prelude::*;
 
 mod prelude {
     pub(crate) use super::*;
-    pub(crate) use index::Simple as SimpleIndex;
+    pub(crate) use index::{FilterResult, Index};
     pub(crate) use std::sync::atomic::AtomicU64;
 }
