@@ -7,13 +7,16 @@ mod observer_worker;
 pub use self::{
     builder::Builder,
     core::{Key, Storage},
+    observer::ActiveBlobPred,
+    observer::ActiveBlobStat,
 };
 
 mod prelude {
     pub(crate) use {
         super::{
             config::Config, core::Inner, observer::Msg, observer::Observer,
-            observer_worker::ObserverWorker,
+            observer::OperationType, observer_worker::ObserverWorker, ActiveBlobPred,
+            ActiveBlobStat,
         },
         crate::prelude::*,
     };
